#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# see benchsimple.py
# manually written python code (for Python3)
#
# probably nearly fastest possible...
# Note: escape consumes about 1/3..1/2 of the time

import time, sys
from benchsimple import test_results, context_dict, N

#----------------------
# evaluation
cache={}

def _eval(expr, a=None):
    if expr not in cache:
        cache[expr] = compile(expr,"","eval")
    return eval(cache[expr],{"__builtins__":None},nsDict)

#----------------------
# different escapeing-routines

(HTML, LATEX) = range(0, 2)
#import cgi
#import xml.sax.saxutils
#from xml.sax.saxutils import escape,quoteattr
def escape(s, format=HTML):
#    return cgi.escape(s, True)     #is slower
#    return xml.sax.saxutils.escape(s)   #s
#    return xml.sax.saxutils.escape(s,{'"':"&quot;","'":"&abc;"})   #s
    s = s.replace("&", "&amp;") # must be done first!
    s = s.replace("<", "&lt;")
    s = s.replace(">", "&gt;")
    s = s.replace('"', "&quot;")
    s = s.replace("'", "&#39;")
    return s

#escape = escape_                       #0.106..8, 0.0675..69, <>&"'
#escape = cgi.escape                    #0.092..4, 0.061..2    <>&
#escape = xml.sax.saxutils.escape       #0.095..6, 0.063..67   <>&
                                        #cgi..., xml... are both slower if wrapped into escape-function
myDict = context_dict.copy()

#----------------------
# different handcoded versions

result_header = '''<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
          "http://www.w3.org/TR/html4/loose.dtd">
<html>

<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
    <style type="text/css">
	tr.row1 { background-color: #00ffff; }
	tr.row2 { background-color: #ffff00; }
	td.red  { background-color: #ff0000; }
    </style>

'''

def handcoded_eval_escape(nsDict):
    myDict.update(nsDict)
    l = []
    write = l.append
    write(result_header)
    write('''    <title>Benchsimple | ''')
    write(escape(_eval("title")))
    write('''</title>
</head>

<body>
    <h1>Benchsimple | ''')
    write(escape(_eval("title")))
    write('''</h1>
    <ul id="nav">
''')
    for i in enumerate(_eval("navigation")):
        nsDict["n"]    = i[0]
        nsDict["item"] = i[1]
        write('''        <li><a href="''')
        write(escape(_eval('''item["href"]''')))
        write('''" id="btn-''')
        write(str(_eval('''n+1''')))
        write('''">''')
        write(escape(_eval('''item["caption"]''')))
        write("</a></li>\n")
    write( '''    </ul>

    <table>
''' )
    for i in enumerate(_eval("table")):
        nsDict["n"]  =i[0]
        nsDict["row"]=i[1]
        write( '''        <tr class=\"''' )
        if _eval('''n%2'''):
            write( '''row2''' )
        else:
            write( '''row1''' )
        write( '''\">
''' )
        for col in _eval("row"):
            nsDict["col"]=col
            if col == nsDict["highlight"]:
                write( '''            <td class="red">''' )
            else:
                write( '''            <td>''' )
            write( escape(str(_eval("col"))) )
            write( '''</td>\n''' )
        write( '''        </tr>\n''' )
    write( '''    </table>

</body>
</html>
''' )
    return "".join(l)


def handcoded_eval_noescape(nsDict):
    myDict.update(nsDict)
    l = []
    write = l.append
    write(result_header)
    write('''    <title>Benchsimple | ''')
    write(_eval("title"))
    write('''</title>
</head>

<body>
    <h1>Benchsimple | ''')
    write(_eval("title"))
    write('''</h1>
    <ul id="nav">
''')
    for i in enumerate(_eval("navigation")):
        nsDict["n"]    = i[0]
        nsDict["item"] = i[1]
        write( '''        <li><a href="''' )
        write( _eval('''item["href"]''') )
        write( '''" id="btn-''' )
        write( str(_eval('''n+1''')) )
        write( '''">''' )
        write( _eval('''item["caption"]''') )
        write( "</a></li>\n" )
    write( '''    </ul>

    <table>
''' )
    for i in enumerate(_eval("table")):
        nsDict["n"]  =i[0]
        nsDict["row"]=i[1]
        write( '''        <tr class=\"''' )
        if _eval('''n%2'''):
            write( '''row2''' )
        else:
            write( '''row1''' )
        write( '''\">
''' )
        for col in _eval("row"):
            nsDict["col"]=col
            if col == nsDict["highlight"]:
                write( '''            <td class="red">''' )
            else:
                write( '''            <td>''' )
            write( str(_eval("col")) )
            write( '''</td>\n''' )
        write( '''        </tr>\n''' )
    write( '''    </table>

</body>
</html>
''' )
    return "".join(l)


def handcoded_noeval_escape(nsDict):
    myDict.update(nsDict)
    l = []
    write = l.append
    write(result_header)
    write('''    <title>Benchsimple | ''')
    write(escape(nsDict["title"]))
    write('''</title>
</head>

<body>
    <h1>Benchsimple | ''')
    write(escape(nsDict["title"]))
    write('''</h1>
    <ul id="nav">
''')
    for i in enumerate(nsDict["navigation"]):
        nsDict["n"]    = i[0]
        nsDict["item"] = i[1]
        write('''        <li><a href="''')
        write(escape(nsDict["item"]["href"]))
        write('''" id="btn-''')
        write(str(nsDict["n"]+1))
        write('''">''')
        write(escape(nsDict["item"]["caption"]))
        write("</a></li>\n")
    write( '''    </ul>

    <table>
''' )
    for i in enumerate(nsDict["table"]):
        nsDict["n"]  =i[0]
        nsDict["row"]=i[1]
        write( '''        <tr class=\"''' )
        if nsDict["n"]%2:
            write( '''row2''' )
        else:
            write( '''row1''' )
        write( '''\">
''' )
        for col in nsDict["row"]:
            nsDict["col"]=col
            if col == nsDict["highlight"]:
                write( '''            <td class="red">''' )
            else:
                write( '''            <td>''' )
            write( escape(str(nsDict["col"])) )
            write( '''</td>\n''' )
        write( '''        </tr>\n''' )
    write( '''    </table>

</body>
</html>
''' )
    return "".join(l)


def handcoded_noeval_noescape(nsDict):
    myDict.update(nsDict)
    l = []
    write = l.append
    write(result_header)
    write('''    <title>Benchsimple | ''')
    write(nsDict["title"])
    write('''</title>
</head>

<body>
    <h1>Benchsimple | ''')
    write(nsDict["title"])
    write('''</h1>
    <ul id="nav">
''')
    for i in enumerate(nsDict["navigation"]):
        nsDict["n"]    = i[0]
        nsDict["item"] = i[1]
        write( '''        <li><a href="''' )
        write( nsDict["item"]["href"] )
        write( '''" id="btn-''' )
        write( str(nsDict["n"]+1) )
        write( '''">''' )
        write( nsDict["item"]["caption"] )
        write( "</a></li>\n" )
    write( '''    </ul>

    <table>
''' )
    for i in enumerate(nsDict["table"]):
        nsDict["n"]  =i[0]
        nsDict["row"]=i[1]
        write( '''        <tr class=\"''' )
        if nsDict["n"]%2:
            write( '''row2''' )
        else:
            write( '''row1''' )
        write( '''\">
''' )
        for col in nsDict["row"]:
            nsDict["col"]=col
            if col == nsDict["highlight"]:
                write( '''            <td class="red">''' )
            else:
                write( '''            <td>''' )
            write( str(nsDict["col"]) )
            write( '''</td>\n''' )
        write( '''        </tr>\n''' )
    write( '''    </table>

</body>
</html>
''' )
    return "".join(l)


#-----------------------------------------
# timer
def timer(name="----", reset=False, overall=False, N=1, _=[]):
    """Simple timer: Print the time elapsed between its calls to stderr.

    :Example:
        >>> import sys;sys.stderr=sys.stdout    # ignore this

        >>> timer()             # init the timer
        >>> l=[i*2 for i in range(1,1000)]
        >>> timer("mytest")                         # doctest:+ELLIPSIS
        ### mytest                          ... s
        >>> timer()                                 # doctest:+ELLIPSIS
        ### ----                            ... s
        >>> time.sleep(1.01)
        >>> timer("sleep 1.01")                     # doctest:+ELLIPSIS
        ### sleep 1.01                      ... s
    """
    if _ == []:
        _.append(time.time())
        _.append(_[0])
    elif overall is True:
        print("### %-30s  %8.6lf s" % (name,(time.time()-_[0])), file=sys.stderr)
        _[1] = time.time()
    else:
        print("### %-30s  %8.6lf s" % (name,(time.time()-_[1])), file=sys.stderr)
        _[1] = time.time()
    if reset:
        _[0] = _[1] = time.time()

#-----------------------------------------
nsDict = context_dict

print("-- for %s iterations --" % N)

timer()
for i in range(N):
    r1 = handcoded_eval_escape(nsDict)
timer("handcoded,   eval   escape:", N)

for i in range(N):
    r2 = handcoded_noeval_escape(nsDict)
timer("handcoded, noeval   escape:", N)

for i in range(N):
    r3 = handcoded_eval_noescape(nsDict)
timer("handcoded,   eval noescape:", N)

for i in range(N):
    r4 = handcoded_noeval_noescape(nsDict)
timer("handcoded, noeval noescape:", N)

print("handcoded, rk, 2013-04-01")
test_results(r1, r2, r3, r4)
print("Note: 2 are wrong because of 'noescape'")

#-----------------------------------------

